@@ -8,7 +8,13 @@ from emmet.core.utils import jsanitize
 
 
 class ChemEnvBuilder(MapBuilder):
-    def __init__(self, oxidation_states: Store, chemenv: Store, query: Optional[Dict] = None, **kwargs):
+    def __init__(
+        self,
+        oxidation_states: Store,
+        chemenv: Store,
+        query: Optional[Dict] = None,
+        **kwargs
+    ):
         self.oxidation_states = oxidation_states
         self.chemenv = chemenv
         self.kwargs = kwargs
