@@ -3,6 +3,7 @@ from maggma.stores import JSONStore, MemoryStore
 
 from emmet.builders.qchem.molecules import MoleculesAssociationBuilder, MoleculesBuilder
 from emmet.builders.molecules.redox import RedoxBuilder
+from emmet.builders.molecules.thermo import ThermoBuilder
 
 
 __author__ = "Evan Spotte-Smith <ewcspottesmith@lbl.gov>"
@@ -20,7 +21,7 @@ def mol_store(tasks_store):
     stage_one.run()
 
     mol_store = MemoryStore(key="molecule_id")
-    stage_two = MoleculesBuilder(assoc=assoc_store, molecules=mol_store, prefix="libe")
+    stage_two = MoleculesBuilder(assoc=assoc_store, molecules=mol_store)
     stage_two.run()
 
     return mol_store
@@ -31,10 +32,17 @@ def redox_store():
     return MemoryStore()
 
 
-@pytest.mark.skip(reason="Waiting on molecule update.")
-def test_redox_builder(mol_store, redox_store):
-    builder = RedoxBuilder(mol_store, redox_store)
+@pytest.fixture(scope="session")
+def thermo_store():
+    return MemoryStore()
+
+
+def test_redox_builder(tasks_store, mol_store, thermo_store, redox_store):
+    thermo_builder = ThermoBuilder(tasks_store, mol_store, thermo_store)
+    thermo_builder.run()
+
+    builder = RedoxBuilder(tasks_store, mol_store, thermo_store, redox_store)
     builder.run()
 
-    assert redox_store.count() == 15
+    assert redox_store.count() == 42
     assert redox_store.count({"deprecated": True}) == 0
